"""Run queries against database specified in the .env file and generate output report file."""
import pyodbc
import os
from dotenv import load_dotenv

import os
import sys
import click
import pathlib
import logging
import pathlib
import yaml

from datetime import datetime
from rich.console import Console
from typing import Dict


from database_reporting_utils import constants
from database_reporting_utils.file_utils import check_infile_status


DEFAULT_OUTDIR = os.path.join(
    constants.DEFAULT_OUTDIR_BASE,
    os.path.splitext(os.path.basename(__file__))[0],
    constants.DEFAULT_TIMESTAMP
)

error_console = Console(stderr=True, style="bold red")

console = Console()



def load_db_config() -> Dict[str, str]:
    """Load database configuration from .env file."""
    check_infile_status(".env")

    logging.info(f"Will load database configuration from .env file")

    load_dotenv()

    return {
        "server": os.getenv("DB_SERVER"),
        "database": os.getenv("DB_DATABASE"),
        "username": os.getenv("DB_USERNAME"),
        "password": os.getenv("DB_PASSWORD"),
        "driver": os.getenv("DB_DRIVER", "ODBC Driver 17 for SQL Server")
    }

def execute_queries_from_file(
        logfile: str,
        sql_file: str,
        report_file: str,
        verbose=True
    ) -> None:
    """Execute SQL queries from a file and write the results to a report file."""
    db_config = load_db_config()

    connection_string = (
        f'DRIVER={{{db_config["driver"]}}};'
        f'SERVER={db_config["server"]};'
        f'DATABASE={db_config["database"]};'
        f'UID={db_config["username"]};'
        f'PWD={db_config["password"]};'
    )

    logging.info(f"Connection string is '{connection_string}'")

    try:
        conn = pyodbc.connect(connection_string)
        cursor = conn.cursor()
        logging.info(f"Connected to database '{db_config['database']}' on server '{db_config['server']}'")
        logging.info(f"Reading SQL queries from file '{sql_file}'")
        with open(sql_file, 'r') as f, open(report_file, 'w') as report:
            report.write(f"## method-created: {os.path.abspath(__file__)}\n")
            report.write(f"## date-created: {datetime.today().strftime('%Y-%m-%d-%H%M%S')}\n")
            report.write(f"## created-by: {os.environ.get('USER')}\n")
            report.write(f"## infile: {sql_file}\n")
            report.write(f"## logfile: {logfile}\n")
            query = ""

            query_ctr = 0

            for line in f:
                line = line.strip()
                if line.startswith("--"):
                    report.write(f"{line}\n")
                    continue  # Ignore comment lines
                query += " " + line
                if line.endswith(";"):
                    query = query.strip()
                    query_ctr += 1
                    logging.info(f"Processing query {query_ctr}: {query}")
                    report.write(f"Query {query_ctr}: {query}\n")
                    if verbose:
                        print(f"Executing Query {query_ctr}: {query}\n")
                    try:
                        cursor.execute(query)
                        if cursor.description:
                            columns = [column[0] for column in cursor.description]
                            rows = cursor.fetchall()
                            report.write(f"Results {query_ctr}:\n")
                            report.write("\t".join(columns) + "\n")
                            for row in rows:
                                report.write("\t".join(str(col) for col in row) + "\n")
                        else:
                            conn.commit()
                            report.write(f"Query {query_ctr} executed successfully.\n")
                    except Exception as e:
                        report.write(f"Error: {e}\n")
                    report.write("\n" + "-" * 50 + "\n\n")
                    query = ""  # Reset query buffer
        cursor.close()
        conn.close()
        logging.info("Closed database connection")
        logging.info(f"Results written to file '{report_file}'")
        print(f"Wrote query results to '{report_file}'")
    except Exception as e:
        print(f"Database connection error: {e}")
        return


def validate_verbose(ctx, param, value):
    """Validate the validate option.

    Args:
        ctx (Context): The click context.
        param (str): The parameter.
        value (bool): The value.

    Returns:
        bool: The value.
    """

    if value is None:
        click.secho("--verbose was not specified and therefore was set to 'True'", fg='yellow')
        return constants.DEFAULT_VERBOSE
    return value


@click.command()
@click.option('--infile', help="The input file containing the SQL queries.")
@click.option('--logfile', help="The log file.")
@click.option('--outdir', help=f"The default is the current working directory - default is '{DEFAULT_OUTDIR}'.")
@click.option('--outfile', help="The output final report file.")
@click.option('--verbose', is_flag=True, help=f"Will print more info to STDOUT - default is '{constants.DEFAULT_VERBOSE}'.", callback=validate_verbose)
def main(
    infile: str,
    logfile: str,
    outdir: str,
    outfile: str,
    verbose: bool
):
    """Run queries against database specified in the .env file and generate output report file."""
    error_ctr = 0

    if infile is None:
        error_console.print("--infile was not specified")
        error_ctr += 1

    if error_ctr > 0:
        click.echo(click.get_current_context().get_help())
        sys.exit(1)

    check_infile_status(infile)

    infile = os.path.abspath(infile)

    if outdir is None:
        outdir = DEFAULT_OUTDIR
        console.print(f"[yellow]--outdir was not specified and therefore was set to '{outdir}'[/]")

    if not os.path.exists(outdir):
        pathlib.Path(outdir).mkdir(parents=True, exist_ok=True)
        console.print(f"[yellow]Created output directory '{outdir}'[/]")

    if logfile is None:
        logfile = os.path.join(
            outdir,
            os.path.splitext(os.path.basename(__file__))[0] + '.log'
        )
        console.print(f"[yellow]--logfile was not specified and therefore was set to '{logfile}'[/]")

    logfile = os.path.abspath(logfile)

    if outfile is None:
        outfile = os.path.join(
            outdir,
            os.path.splitext(os.path.basename(__file__))[0] + '.query_results.txt'
        )
        console.print(f"[yellow]--outfile was not specified and therefore was set to '{outfile}'[/]")

    if verbose is None:
        verbose = constants.DEFAULT_VERBOSE
        console.print(f"[yellow]--verbose was not specified and therefore was set to '{verbose}'[/]")

    logging.basicConfig(
        filename=logfile,
        format=constants.DEFAULT_LOGGING_FORMAT,
        level=constants.DEFAULT_LOGGING_LEVEL,
    )

    execute_queries_from_file(
        logfile,
        infile,
        outfile,
        verbose
    )

    if verbose:
        print(f"The log file is '{logfile}'")
        console.print(f"[bold green]Execution of '{os.path.abspath(__file__)}' completed[/]")


if __name__ == "__main__":
    main()
